package checkers.quals;

import java.lang.annotation.Annotation;
import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * A meta annotation to specify all the qualifiers that the given qualifier
 * is a subtype of.
 *
 * <p>
 * Example:
 * <code>@SubtypeOf( { Nullable.class } )
 * public @interface NonNull { }
 * </code>
 *
 * <p>
 *
 * As a special case, an empty set of values indicate the root type qualifier
 * in the qualifiers hierarchy, i.e. the qualifier that is a supertype of all
 * other qualifiers in the given hierarchy.
 *
 * Examples of such qualifiers: {@code @ReadOnly}, {@code @Nullable}
 *
 * <p>
 *
 * <code>@SubtypeOf( {} )
 * public @interface ReadOnly { }
 * </code>
 *
 */
@Documented
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.ANNOTATION_TYPE)
public @interface SubtypeOf {
    /** An array of the  supertype qualifiers of the annotated qualifier **/
    Class<? extends Annotation>[] value();
}
